@@ -2,7 +2,7 @@ use strictures;
 use 5.14.0;
 
 package WebService::GoogleAPI::Client;
-$WebService::GoogleAPI::Client::VERSION = '0.16';
+$WebService::GoogleAPI::Client::VERSION = '0.17';
 
 # ABSTRACT: API WebService OAUTH2 Client Agent to streamline access to GOOGLE API End-Point Services using Discovery Data
 
@@ -292,15 +292,11 @@ sub has_scope_to_access_api_endpoint
 ########################################################
 
 
-
-
-#=head2 MANUAL API REQUEST CONSTRUCTION
-#    ## Completely manually constructed API End-Point Request to obtain Perl Data Structure converted from JSON response.
-#    my $res = $gapi_client->api_query(
-#      method => 'get',
-#      path => 'https://www.googleapis.com/calendar/users/me/calendarList',
-#    )->json;
+#TODO: Consider rename to return_fetched_google_v1_apis_discovery_structure
 #
+#TODO - handle auth required error and resubmit request with OAUTH headers if response indicates
+#       access requires auth ( when exceed free access limits )
+
 
 1;
 
@@ -316,7 +312,7 @@ WebService::GoogleAPI::Client - API WebService OAUTH2 Client Agent to streamline
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 SYNOPSIS
 
@@ -337,11 +333,15 @@ redentials and user authorization created by _goauth_
       say 'User has Access to GMail Method End-Point gmail.users.settings.sendAs.get';
     }
 
-Package includes _go_auth_ CLI Script to collect initial end-user authorisation to scoped services
+Internal User Agent provided be property WebService::GoogleAPI::Client::UserAgent dervied from Mojo::UserAgent
 
-=for html <a href="https://travis-ci.org/pscott-au//WebService-GoogleAPI-Client"><img alt="Build Status" src="https://api.travis-ci.org/pscott-au/WebService-GoogleAPI-Client.png?branch=master" /></a>
+Package includes I<go_auth> CLI Script to collect initial end-user authorisation to scoped services
+
+=begin html <a href="https://travis-ci.org/pscott-au//WebService-GoogleAPI-Client"><img alt="Build Status" src="https://api.travis-ci.org/pscott-au/WebService-GoogleAPI-Client.png?branch=master" /></a>
 <a href="https://metacpan.org/pod/WebService::GoogleAPI::Client"><img alt="CPAN version" src="https://img.shields.io/cpan/v/WebService-GoogleAPI-Client.png" /></a>
 
+=end html
+
 =head1 EXAMPLES
 
 =head2 AUTOMATIC API REQUEST CONSTRUCTION  - SEND EMAL
@@ -376,7 +376,17 @@ Package includes _go_auth_ CLI Script to collect initial end-user authorisation
 
 =head2 C<new>
 
-WebService::GoogleAPI::Client->new( user => 'useremail@sdf.com', gapi_json => '/fullpath/gapi.json' );
+  WebService::GoogleAPI::Client->new( user => 'peter@pscott.com.au', gapi_json => '/fullpath/gapi.json' );
+
+=head3 PARAMETERS
+
+=head4 user :: the email address that identifies key of credentials in the config file
+
+=head4 gapi_json :: Location of the configuration credentials - default gapi.json
+
+=head4 debug :: if '1' then diagnostics are send to STDERR - default false
+
+=head4 chi :: an instance to a CHI persistent storage case object - if none provided FILE is used
 
 =head2 C<api_query>
 
@@ -390,8 +400,6 @@ Optional params: api_endpoint_id
 
 $self->access_token must be valid
 
-Examples of usage:
-
   $gapi->api_query({
       method => 'get',
       path => 'https://www.googleapis.com/calendar/users/me/calendarList',
@@ -437,49 +445,109 @@ Returns 0 if scope to access is not available to the user.
 
 warns and returns 0 on error ( eg user or config not specified etc )
 
+=head1 METHODS DELEGATED TO WebService::GoogleAPI::Client::Discovery
+
+=head2 C<discover_all>
+
+  Return details about all Available Google APIs as provided by Google or in CHI Cache
+
+  On Success: Returns HASHREF containing items key => list of hashes describing each API
+  On Failure: Warns and returns empty hashref
+
+    my $client = WebService::GoogleAPI::Client->new; ## has discovery member WebService::GoogleAPI::Client::Discovery
+
+    $d = $client->discover_all();
+    $d = $client->discover_all(1); ## NB if include a parameter that evaluates to true such as '1' then the cache is flushed with a new version
+
+    ## OR
+    $d = $client->discovery-> discover_all();
+    $d = WebService::GoogleAPI::Client::Discovery->discover_all();
+
+    print Dumper $d;
+
+      $VAR1 = {
+                'items' => [
+                            {
+                              'preferred' => bless( do{\(my $o = 1)}, 'JSON::PP::Boolean' ),
+                              'id' => 'abusiveexperiencereport:v1',
+                              'icons' => {
+                                            'x32' => 'https://www.gstatic.com/images/branding/product/1x/googleg_32dp.png',
+                                            'x16' => 'https://www.gstatic.com/images/branding/product/1x/googleg_16dp.png'
+                                          },
+                              'version' => 'v1',
+                              'documentationLink' => 'https://developers.google.com/abusive-experience-report/',
+                              'kind' => 'discovery#directoryItem',
+                              'discoveryRestUrl' => 'https://abusiveexperiencereport.googleapis.com/$discovery/rest?version=v1',
+                              'title' => 'Abusive Experience Report API',
+                              'name' => 'abusiveexperiencereport',
+                              'description' => 'Views Abusive Experience Report data, and gets a list of sites that have a significant number of abusive experiences.'
+                            }, ...
+
+    ## NB because the structure isn't indexed on the api name it can be convenient to post-process it
+    ## 
+    
+    my $new_hash = {};
+    foreach my $api ( @{ %{$client->discover_all()}{items} } )
+    {
+        # convert JSON::PP::Boolean to true|false strings
+        $api->{preferred}  = "$api->{preferred}" if defined $api->{preferred};
+        $api->{preferred}  = $api->{preferred} eq '0' ? 'false' : 'true';
+
+        $new_hash->{ $api->{name} } = $api;
+    }
+    print Dumper $new_hash->{gmail};
+
+=head2 C<get_api_discovery_for_api_id>
+
+returns the cached version if avaiable in CHI otherwise retrieves discovery data via HTTP, stores in CHI cache and returns as
+a Perl data structure.
+
+    my $hashref = $self->get_api_discovery_for_api_id( 'gmail' );
+    my $hashref = $self->get_api_discovery_for_api_id( 'gmail:v3' );
+
+returns the api discovery specification structure ( cached by CHI ) for api id ( eg 'gmail ')
+
+returns the discovery data as a hashref, an empty hashref on certain failing conditions or croaks on critical errors.
+
 =head2 C<methods_available_for_google_api_id>
 
 Returns a hashref keyed on the Google service API Endpoint in dotted format.
-The hashed content contains a structure
-representing the corresponding discovery specification for that method ( API Endpoint )
+The hashed content contains a structure  representing the corresponding 
+discovery specification for that method ( API Endpoint ).
 
     methods_available_for_google_api_id('gmail')
 
-TODO: consider ? refactor to allow parameters either as a single api id such as 'gmail' 
-      as well as the currently accepted  hash keyed on the api and version
+=head2 C<extract_method_discovery_detail_from_api_spec>
 
-DELEGATED FROM WebService::GoogleAPI::Client::Discovery
+    $my $api_detail = $gapi->discovery->extract_method_discovery_detail_from_api_spec( 'gmail.users.settings' );
 
-SEE ALSO:  
-  The following methods are delegated through to Client::Discovery - see perldoc WebService::Client::Discovery for detils
+returns a hashref representing the discovery specification for the method identified by $tree in dotted API format such as texttospeech.text.synthesize
 
-  get_method_meta 
-  discover_all 
-  extract_method_discovery_detail_from_api_spec 
-  get_api_discovery_for_api_id
+returns an empty hashref if not found
 
 =head2 C<list_of_available_google_api_ids>
 
 Returns an array list of all the available API's described in the API Discovery Resource
 that is either fetched or cached in CHI locally for 30 days.
 
-WHen called in a scalar context returns the list as a comma joined string.
+    my $r = $agent->list_of_available_google_api_ids();
+    print "List of API Services ( comma separated): $r\n";
 
-DELEGATED FROM WebService::GoogleAPI::Client::Discovery
+    my @list = $agent->list_of_available_google_api_ids();
 
 =head1 FEATURES
 
 =over 1
 
-=item API Discovery with local caching using CHI File
+=item * API Discovery requests cached with CHI ( Default File )
 
-=item OAUTH app credentials (client_id, client_secret, scope, users access_token and refresh_tokens) stored in local file (default name =  gapi.json)
+=item * OAUTH app and user credentials (client_id, client_secret, scope, users access_token and refresh_tokens) stored in local file (default name =  gapi.json)
 
-=item access_token refreshes when expires (if user has refresh_token) saving refreshed token back to json file
+=item * access_token auto-refreshes when expires (if user has refresh_token) saving refreshed token back to json file
 
-=item helper api_query to streamline request composition without preventing manual construction if preferred.
+=item * helper api_query to streamline request composition without preventing manual construction if preferred.
 
-=item CLI tool (I<goauth>) with lightweight http server to simplify OAuth2 configuration, sccoping, authorization and obtaining access_ and refresh_ tokensn from users
+=item * CLI tool (I<goauth>) with lightweight HTTP server to simplify OAuth2 configuration, sccoping, authorization and obtaining access_ and refresh_ tokens from users
 
 =back
 
